@@ -41,22 +41,25 @@ sub update_all {
 sub update ( $type, $cv = undef ) {
     state $init = !!require IO::Uncompress::Gunzip;
 
+    $cv->begin if $cv;
+
     P->http->get(
         $RES->{$type}->[1],
         buf_size    => 1,
         on_progress => 1,
-        cv          => $cv,
         on_finish   => sub ($res) {
             if ( $res->status == 200 ) {
                 my $temp = P->file->tempfile;
 
                 IO::Uncompress::Gunzip::gunzip( $res->body, $temp->path, BinModeOut => 1 );
 
-                $ENV->share->store( $temp->path, $RES->{$type}->[0], 'Pcore-GeoIP' );
+                $ENV->share->store( $RES->{$type}->[0], $temp->path, 'Pcore-GeoIP' );
 
                 delete $H->{$type};
             }
 
+            $cv->end if $cv;
+
             return;
         }
     );
